@@ -37,9 +37,15 @@ import (
 	_ "k8s.io/kube-openapi/cmd/openapi-gen"
 	_ "knative.dev/pkg/codegen/cmd/injection-gen"
 
+	// For chaos testing the leaderelection stuff.
+	_ "knative.dev/pkg/leaderelection/chaosduck"
+
 	// caching resource
 	_ "knative.dev/caching/config"
 
+	// networking resource
+	_ "knative.dev/networking/config"
+
 	// For load testing
 	_ "github.com/tsenart/vegeta"
 	_ "knative.dev/pkg/testutils/clustermanager/perf-tests"
