@@ -1,5 +1,5 @@
 /*
-Copyright 2018 The Knative Authors.
+Copyright 2018 The Knative Authors
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -40,7 +40,7 @@ import (
 	defaultconfig "knative.dev/serving/pkg/apis/config"
 	"knative.dev/serving/pkg/apis/serving"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
-	asconfig "knative.dev/serving/pkg/autoscaler/config"
+	"knative.dev/serving/pkg/autoscaler/config/autoscalerconfig"
 	servingclient "knative.dev/serving/pkg/client/injection/client"
 	revisionreconciler "knative.dev/serving/pkg/client/injection/reconciler/serving/v1/revision"
 	"knative.dev/serving/pkg/reconciler/revision/config"
@@ -66,7 +66,7 @@ func TestReconcile(t *testing.T) {
 		Name: "nop deletion reconcile",
 		// Test that with a DeletionTimestamp we do nothing.
 		Objects: []runtime.Object{
-			rev("foo", "delete-pending", WithRevisionDeletionTimestamp),
+			Revision("foo", "delete-pending", WithRevisionDeletionTimestamp),
 		},
 		Key: "foo/delete-pending",
 	}, {
@@ -75,7 +75,7 @@ func TestReconcile(t *testing.T) {
 		// We feed in a well formed Revision where none of its sub-resources exist,
 		// and we expect it to create them and initialize the Revision's status.
 		Objects: []runtime.Object{
-			rev("foo", "first-reconcile"),
+			Revision("foo", "first-reconcile"),
 		},
 		WantCreates: []runtime.Object{
 			// The first reconciliation of a Revision creates the following resources.
@@ -84,10 +84,10 @@ func TestReconcile(t *testing.T) {
 			image("foo", "first-reconcile"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "first-reconcile",
+			Object: Revision("foo", "first-reconcile",
 				// The first reconciliation Populates the following status properties.
-				WithLogURL, AllUnknownConditions, MarkDeploying("Deploying"),
-				withDefaultContainerStatuses(), withObservedGeneration(1)),
+				WithLogURL, allUnknownConditions, MarkDeploying("Deploying"),
+				withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		Key: "foo/first-reconcile",
 	}, {
@@ -99,7 +99,7 @@ func TestReconcile(t *testing.T) {
 			InduceFailure("update", "revisions"),
 		},
 		Objects: []runtime.Object{
-			rev("foo", "update-status-failure"),
+			Revision("foo", "update-status-failure"),
 			pa("foo", "update-status-failure"),
 		},
 		WantCreates: []runtime.Object{
@@ -108,10 +108,10 @@ func TestReconcile(t *testing.T) {
 			image("foo", "update-status-failure"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "update-status-failure",
+			Object: Revision("foo", "update-status-failure",
 				// Despite failure, the following status properties are set.
-				WithLogURL, AllUnknownConditions, MarkDeploying("Deploying"),
-				withDefaultContainerStatuses(), withObservedGeneration(1)),
+				WithLogURL, allUnknownConditions, MarkDeploying("Deploying"),
+				withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "UpdateFailed", "Failed to update status for %q: %v",
@@ -127,7 +127,7 @@ func TestReconcile(t *testing.T) {
 			InduceFailure("create", "podautoscalers"),
 		},
 		Objects: []runtime.Object{
-			rev("foo", "create-pa-failure"),
+			Revision("foo", "create-pa-failure"),
 		},
 		WantCreates: []runtime.Object{
 			// We still see the following creates before the failure is induced.
@@ -136,10 +136,10 @@ func TestReconcile(t *testing.T) {
 			image("foo", "create-pa-failure"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "create-pa-failure",
+			Object: Revision("foo", "create-pa-failure",
 				// Despite failure, the following status properties are set.
 				WithLogURL, WithInitRevConditions,
-				MarkDeploying("Deploying"), withDefaultContainerStatuses(), withObservedGeneration(1)),
+				MarkDeploying("Deploying"), withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "InternalError", `failed to create PA "create-pa-failure": inducing failure for create podautoscalers`),
@@ -154,7 +154,7 @@ func TestReconcile(t *testing.T) {
 			InduceFailure("create", "deployments"),
 		},
 		Objects: []runtime.Object{
-			rev("foo", "create-user-deploy-failure"),
+			Revision("foo", "create-user-deploy-failure"),
 			pa("foo", "create-user-deploy-failure"),
 		},
 		WantCreates: []runtime.Object{
@@ -162,10 +162,10 @@ func TestReconcile(t *testing.T) {
 			deploy(t, "foo", "create-user-deploy-failure"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "create-user-deploy-failure",
+			Object: Revision("foo", "create-user-deploy-failure",
 				// Despite failure, the following status properties are set.
 				WithLogURL, WithInitRevConditions,
-				MarkDeploying("Deploying"), withDefaultContainerStatuses(), withObservedGeneration(1)),
+				MarkDeploying("Deploying"), withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "InternalError",
@@ -179,8 +179,8 @@ func TestReconcile(t *testing.T) {
 		// state (immediately post-creation), and verify that no changes
 		// are necessary.
 		Objects: []runtime.Object{
-			rev("foo", "stable-reconcile", WithLogURL, AllUnknownConditions,
-				withDefaultContainerStatuses(), withObservedGeneration(1)),
+			Revision("foo", "stable-reconcile", WithLogURL, allUnknownConditions,
+				withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 			pa("foo", "stable-reconcile", WithReachabilityUnknown),
 
 			deploy(t, "foo", "stable-reconcile"),
@@ -193,8 +193,8 @@ func TestReconcile(t *testing.T) {
 		// Test that we update a deployment with new containers when they disagree
 		// with our desired spec.
 		Objects: []runtime.Object{
-			rev("foo", "fix-containers",
-				WithLogURL, AllUnknownConditions, withDefaultContainerStatuses(), withObservedGeneration(1)),
+			Revision("foo", "fix-containers",
+				WithLogURL, allUnknownConditions, withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 			pa("foo", "fix-containers", WithReachabilityUnknown),
 			changeContainers(deploy(t, "foo", "fix-containers")),
 			image("foo", "fix-containers"),
@@ -211,9 +211,9 @@ func TestReconcile(t *testing.T) {
 			InduceFailure("update", "deployments"),
 		},
 		Objects: []runtime.Object{
-			rev("foo", "failure-update-deploy",
-				withK8sServiceName("whateves"), WithLogURL, AllUnknownConditions,
-				withDefaultContainerStatuses(), withObservedGeneration(1)),
+			Revision("foo", "failure-update-deploy",
+				WithK8sServiceName("whateves"), WithLogURL, allUnknownConditions,
+				withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 			pa("foo", "failure-update-deploy"),
 			changeContainers(deploy(t, "foo", "failure-update-deploy")),
 			image("foo", "failure-update-deploy"),
@@ -232,12 +232,13 @@ func TestReconcile(t *testing.T) {
 		// We feed in a Revision and the resources it controls in a steady
 		// state (port-Reserve), and verify that no changes are necessary.
 		Objects: []runtime.Object{
-			rev("foo", "stable-deactivation",
+			Revision("foo", "stable-deactivation",
 				WithLogURL, MarkRevisionReady,
 				MarkInactive("NoTraffic", "This thing is inactive."),
-				withDefaultContainerStatuses(), withObservedGeneration(1)),
+				withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 			pa("foo", "stable-deactivation",
-				WithNoTraffic("NoTraffic", "This thing is inactive."), WithReachabilityUnreachable),
+				WithNoTraffic("NoTraffic", "This thing is inactive."), WithReachabilityUnreachable,
+				WithScaleTargetInitialized),
 			deploy(t, "foo", "stable-deactivation"),
 			image("foo", "stable-deactivation"),
 		},
@@ -245,18 +246,19 @@ func TestReconcile(t *testing.T) {
 	}, {
 		Name: "pa is ready",
 		Objects: []runtime.Object{
-			rev("foo", "pa-ready",
-				withK8sServiceName("old-stuff"), WithLogURL, AllUnknownConditions),
-			pa("foo", "pa-ready", WithTraffic, WithPAStatusService("new-stuff"), WithReachabilityUnknown),
+			Revision("foo", "pa-ready",
+				WithK8sServiceName("old-stuff"), WithLogURL, allUnknownConditions),
+			pa("foo", "pa-ready", WithPASKSReady, WithTraffic,
+				WithScaleTargetInitialized, WithPAStatusService("new-stuff"), WithReachabilityUnknown),
 			deploy(t, "foo", "pa-ready"),
 			image("foo", "pa-ready"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "pa-ready", withK8sServiceName("new-stuff"),
+			Object: Revision("foo", "pa-ready", WithK8sServiceName("new-stuff"),
 				WithLogURL,
 				// When the endpoint and pa are ready, then we will see the
 				// Revision become ready.
-				MarkRevisionReady, withDefaultContainerStatuses(), withObservedGeneration(1)),
+				MarkRevisionReady, withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "RevisionReady", "Revision becomes ready upon all resources being ready"),
@@ -266,24 +268,24 @@ func TestReconcile(t *testing.T) {
 		Name: "pa not ready",
 		// Test propagating the pa not ready status to the Revision.
 		Objects: []runtime.Object{
-			rev("foo", "pa-not-ready",
-				withK8sServiceName("somebody-told-me"), WithLogURL,
-				MarkRevisionReady, withObservedGeneration(1)),
+			Revision("foo", "pa-not-ready",
+				WithK8sServiceName("somebody-told-me"), WithLogURL,
+				MarkRevisionReady, WithRevisionObservedGeneration(1)),
 			pa("foo", "pa-not-ready",
 				WithPAStatusService("its-not-confidential"),
-				WithBufferedTraffic("Something", "This is something longer"),
+				WithBufferedTraffic,
 				WithReachabilityUnreachable),
 			readyDeploy(deploy(t, "foo", "pa-not-ready")),
 			image("foo", "pa-not-ready"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "pa-not-ready",
+			Object: Revision("foo", "pa-not-ready",
 				WithLogURL, MarkRevisionReady, withDefaultContainerStatuses(),
-				withK8sServiceName("its-not-confidential"),
+				WithK8sServiceName("its-not-confidential"),
 				// When we reconcile a ready state and our pa is in an activating
 				// state, we should see the following mutation.
-				MarkActivating("Something", "This is something longer"),
-				withObservedGeneration(1),
+				MarkActivating("Queued", "Requests to the target are being buffered as resources are provisioned."),
+				WithRevisionObservedGeneration(1),
 			),
 		}},
 		Key: "foo/pa-not-ready",
@@ -291,21 +293,64 @@ func TestReconcile(t *testing.T) {
 		Name: "pa inactive",
 		// Test propagating the inactivity signal from the pa to the Revision.
 		Objects: []runtime.Object{
-			rev("foo", "pa-inactive",
-				withK8sServiceName("something-in-the-way"), WithLogURL,
-				MarkRevisionReady, withObservedGeneration(1)),
+			Revision("foo", "pa-inactive",
+				WithK8sServiceName("something-in-the-way"), WithLogURL,
+				MarkRevisionReady, WithRevisionObservedGeneration(1)),
 			pa("foo", "pa-inactive",
 				WithNoTraffic("NoTraffic", "This thing is inactive."),
+				WithScaleTargetInitialized,
 				WithReachabilityUnreachable),
 			readyDeploy(deploy(t, "foo", "pa-inactive")),
 			image("foo", "pa-inactive"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "pa-inactive",
+			Object: Revision("foo", "pa-inactive",
 				WithLogURL, MarkRevisionReady, withDefaultContainerStatuses(),
 				// When we reconcile an "all ready" revision when the PA
 				// is inactive, we should see the following change.
-				MarkInactive("NoTraffic", "This thing is inactive."), withObservedGeneration(1)),
+				MarkInactive("NoTraffic", "This thing is inactive."), WithRevisionObservedGeneration(1)),
+		}},
+		Key: "foo/pa-inactive",
+	}, {
+		Name: "pa inactive II",
+		// Test propagating the inactivity signal from the pa to the Revision.
+		Objects: []runtime.Object{
+			Revision("foo", "pa-inactive",
+				WithK8sServiceName("something-in-the-way"), WithLogURL,
+				WithRevisionObservedGeneration(1)),
+			pa("foo", "pa-inactive",
+				WithNoTraffic("NoTraffic", "This thing is inactive."),
+				WithPAStatusService("pa-inactive")),
+			readyDeploy(deploy(t, "foo", "pa-inactive")),
+			image("foo", "pa-inactive"),
+		},
+		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: Revision("foo", "pa-inactive",
+				WithLogURL, withDefaultContainerStatuses(), MarkDeploying(""),
+				// When we reconcile an "all ready" revision when the PA
+				// is inactive, we should see the following change.
+				MarkInactive("NoTraffic", "This thing is inactive."), WithRevisionObservedGeneration(1),
+				MarkResourcesUnavailable(v1.ReasonProgressDeadlineExceeded,
+					"Initial scale was never achieved"), WithK8sServiceName("pa-inactive")),
+		}},
+		Key: "foo/pa-inactive",
+	}, {
+		Name: "pa is not ready with initial scale zero, but ServiceName still empty, so not marking resources available false",
+		Objects: []runtime.Object{
+			Revision("foo", "pa-inactive", allUnknownConditions,
+				WithK8sServiceName("something-in-the-way"), WithLogURL,
+				WithRevisionObservedGeneration(1)),
+			pa("foo", "pa-inactive",
+				WithNoTraffic("NoTraffic", "This thing is inactive."), WithPAStatusService("")),
+			readyDeploy(deploy(t, "foo", "pa-inactive")),
+			image("foo", "pa-inactive"),
+		},
+		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
+			// We should not mark resources unavailable if ServiceName is empty
+			Object: Revision("foo", "pa-inactive",
+				WithLogURL, withDefaultContainerStatuses(), allUnknownConditions,
+				MarkInactive("NoTraffic", "This thing is inactive."),
+				WithRevisionObservedGeneration(1)),
 		}},
 		Key: "foo/pa-inactive",
 	}, {
@@ -313,24 +358,25 @@ func TestReconcile(t *testing.T) {
 		// Test propagating the inactivity signal from the pa to the Revision.
 		// But propagate the service name.
 		Objects: []runtime.Object{
-			rev("foo", "pa-inactive",
-				withK8sServiceName("here-comes-the-sun"), WithLogURL,
-				MarkRevisionReady, withObservedGeneration(1)),
+			Revision("foo", "pa-inactive",
+				WithK8sServiceName("here-comes-the-sun"), WithLogURL,
+				MarkRevisionReady, WithRevisionObservedGeneration(1)),
 			pa("foo", "pa-inactive",
 				WithNoTraffic("NoTraffic", "This thing is inactive."),
 				WithPAStatusService("pa-inactive-svc"),
+				WithScaleTargetInitialized,
 				WithReachabilityUnreachable),
 			readyDeploy(deploy(t, "foo", "pa-inactive")),
 			image("foo", "pa-inactive"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "pa-inactive",
+			Object: Revision("foo", "pa-inactive",
 				WithLogURL, MarkRevisionReady,
-				withK8sServiceName("pa-inactive-svc"),
+				WithK8sServiceName("pa-inactive-svc"),
 				// When we reconcile an "all ready" revision when the PA
 				// is inactive, we should see the following change.
 				MarkInactive("NoTraffic", "This thing is inactive."),
-				withDefaultContainerStatuses(), withObservedGeneration(1)),
+				withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		Key: "foo/pa-inactive",
 	}, {
@@ -339,38 +385,37 @@ func TestReconcile(t *testing.T) {
 		// we bring it back to the required shape.
 		// Protocol type is the only thing that can be changed on PA
 		Objects: []runtime.Object{
-			rev("foo", "fix-mutated-pa",
-				withK8sServiceName("ill-follow-the-sun"), WithLogURL, MarkRevisionReady,
+			Revision("foo", "fix-mutated-pa",
+				WithK8sServiceName("ill-follow-the-sun"), WithLogURL, MarkRevisionReady,
 				WithRevisionLabel(serving.RouteLabelKey, "foo")),
 			pa("foo", "fix-mutated-pa", WithProtocolType(networking.ProtocolH2C),
-				WithTraffic, WithPAStatusService("fix-mutated-pa")),
+				WithTraffic, WithPASKSReady, WithScaleTargetInitialized, WithReachabilityReachable,
+				WithPAStatusService("fix-mutated-pa")),
 			deploy(t, "foo", "fix-mutated-pa"),
 			image("foo", "fix-mutated-pa"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "fix-mutated-pa",
-				WithLogURL, AllUnknownConditions,
+			Object: Revision("foo", "fix-mutated-pa",
+				WithLogURL, allUnknownConditions,
 				// When our reconciliation has to change the service
 				// we should see the following mutations to status.
-				withK8sServiceName("fix-mutated-pa"),
+				WithK8sServiceName("fix-mutated-pa"),
 				WithRevisionLabel(serving.RouteLabelKey, "foo"), WithLogURL, MarkRevisionReady,
-				withDefaultContainerStatuses(), withObservedGeneration(1)),
+				withDefaultContainerStatuses()),
 		}},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: pa("foo", "fix-mutated-pa", WithTraffic,
+			Object: pa("foo", "fix-mutated-pa", WithPASKSReady,
+				WithTraffic, WithScaleTargetInitialized,
 				WithPAStatusService("fix-mutated-pa"), WithReachabilityReachable),
 		}},
-		WantEvents: []string{
-			Eventf(corev1.EventTypeNormal, "RevisionReady", "Revision becomes ready upon all resources being ready"),
-		},
 		Key: "foo/fix-mutated-pa",
 	}, {
 		Name: "mutated pa gets error during the fix",
 		// Same as above, but will fail during the update.
 		Objects: []runtime.Object{
-			rev("foo", "fix-mutated-pa-fail",
-				withK8sServiceName("some-old-stuff"),
-				WithLogURL, AllUnknownConditions, withDefaultContainerStatuses(), withObservedGeneration(1)),
+			Revision("foo", "fix-mutated-pa-fail",
+				WithK8sServiceName("some-old-stuff"),
+				WithLogURL, allUnknownConditions, withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 			pa("foo", "fix-mutated-pa-fail", WithProtocolType(networking.ProtocolH2C), WithReachabilityUnknown),
 			deploy(t, "foo", "fix-mutated-pa-fail"),
 			image("foo", "fix-mutated-pa-fail"),
@@ -394,19 +439,19 @@ func TestReconcile(t *testing.T) {
 		// condition.  It then verifies that Reconcile propagates this into the
 		// status of the Revision.
 		Objects: []runtime.Object{
-			rev("foo", "deploy-timeout",
-				withK8sServiceName("the-taxman"), WithLogURL, MarkActive),
+			Revision("foo", "deploy-timeout",
+				WithK8sServiceName("the-taxman"), WithLogURL, MarkActive),
 			pa("foo", "deploy-timeout"), // pa can't be ready since deployment times out.
 			timeoutDeploy(deploy(t, "foo", "deploy-timeout"), "I timed out!"),
 			image("foo", "deploy-timeout"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "deploy-timeout",
-				WithLogURL, AllUnknownConditions,
+			Object: Revision("foo", "deploy-timeout",
+				WithLogURL, allUnknownConditions,
 				// When the revision is reconciled after a Deployment has
 				// timed out, we should see it marked with the PDE state.
 				MarkProgressDeadlineExceeded("I timed out!"), withDefaultContainerStatuses(),
-				withObservedGeneration(1)),
+				WithRevisionObservedGeneration(1)),
 		}},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: pa("foo", "deploy-timeout", WithReachabilityUnreachable),
@@ -419,19 +464,19 @@ func TestReconcile(t *testing.T) {
 		// but changes the user deployment to have a FailedCreate condition.
 		// It then verifies that Reconcile propagates this into the status of the Revision.
 		Objects: []runtime.Object{
-			rev("foo", "deploy-replica-failure",
-				withK8sServiceName("the-taxman"), WithLogURL, MarkActive),
+			Revision("foo", "deploy-replica-failure",
+				WithK8sServiceName("the-taxman"), WithLogURL, MarkActive),
 			pa("foo", "deploy-replica-failure"),
 			replicaFailureDeploy(deploy(t, "foo", "deploy-replica-failure"), "I replica failed!"),
 			image("foo", "deploy-replica-failure"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "deploy-replica-failure",
-				WithLogURL, AllUnknownConditions,
+			Object: Revision("foo", "deploy-replica-failure",
+				WithLogURL, allUnknownConditions,
 				// When the revision is reconciled after a Deployment has
 				// timed out, we should see it marked with the FailedCreate state.
 				MarkResourcesUnavailable("FailedCreate", "I replica failed!"),
-				withDefaultContainerStatuses(), withObservedGeneration(1)),
+				withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: pa("foo", "deploy-replica-failure", WithReachabilityUnreachable),
@@ -441,17 +486,17 @@ func TestReconcile(t *testing.T) {
 		Name: "surface ImagePullBackoff",
 		// Test the propagation of ImagePullBackoff from user container.
 		Objects: []runtime.Object{
-			rev("foo", "pull-backoff",
-				withK8sServiceName("the-taxman"), WithLogURL, MarkActivating("Deploying", "")),
+			Revision("foo", "pull-backoff",
+				WithK8sServiceName("the-taxman"), WithLogURL, MarkActivating("Deploying", "")),
 			pa("foo", "pull-backoff"), // pa can't be ready since deployment times out.
 			pod(t, "foo", "pull-backoff", WithWaitingContainer("pull-backoff", "ImagePullBackoff", "can't pull it")),
 			timeoutDeploy(deploy(t, "foo", "pull-backoff"), "Timed out!"),
 			image("foo", "pull-backoff"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "pull-backoff",
-				WithLogURL, AllUnknownConditions,
-				MarkResourcesUnavailable("ImagePullBackoff", "can't pull it"), withDefaultContainerStatuses(), withObservedGeneration(1)),
+			Object: Revision("foo", "pull-backoff",
+				WithLogURL, allUnknownConditions,
+				MarkResourcesUnavailable("ImagePullBackoff", "can't pull it"), withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: pa("foo", "pull-backoff", WithReachabilityUnreachable),
@@ -464,17 +509,17 @@ func TestReconcile(t *testing.T) {
 		// but changes the user deployment to have a failing pod. It then verifies
 		// that Reconcile propagates this into the status of the Revision.
 		Objects: []runtime.Object{
-			rev("foo", "pod-error",
-				withK8sServiceName("a-pod-error"), WithLogURL, AllUnknownConditions, MarkActive),
+			Revision("foo", "pod-error",
+				WithK8sServiceName("a-pod-error"), WithLogURL, allUnknownConditions, MarkActive),
 			pa("foo", "pod-error"), // PA can't be ready, since no traffic.
 			pod(t, "foo", "pod-error", WithFailingContainer("pod-error", 5, "I failed man!")),
 			deploy(t, "foo", "pod-error"),
 			image("foo", "pod-error"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "pod-error",
-				WithLogURL, AllUnknownConditions, MarkContainerExiting(5,
-					v1.RevisionContainerExitingMessage("I failed man!")), withDefaultContainerStatuses(), withObservedGeneration(1)),
+			Object: Revision("foo", "pod-error",
+				WithLogURL, allUnknownConditions, MarkContainerExiting(5,
+					v1.RevisionContainerExitingMessage("I failed man!")), withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: pa("foo", "pod-error", WithReachabilityUnreachable),
@@ -486,17 +531,17 @@ func TestReconcile(t *testing.T) {
 		// This initializes the world to unschedule pod. It then verifies
 		// that Reconcile propagates this into the status of the Revision.
 		Objects: []runtime.Object{
-			rev("foo", "pod-schedule-error",
-				withK8sServiceName("a-pod-schedule-error"), WithLogURL, AllUnknownConditions, MarkActive),
+			Revision("foo", "pod-schedule-error",
+				WithK8sServiceName("a-pod-schedule-error"), WithLogURL, allUnknownConditions, MarkActive),
 			pa("foo", "pod-schedule-error"), // PA can't be ready, since no traffic.
 			pod(t, "foo", "pod-schedule-error", WithUnschedulableContainer("Insufficient energy", "Unschedulable")),
 			deploy(t, "foo", "pod-schedule-error"),
 			image("foo", "pod-schedule-error"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "pod-schedule-error",
-				WithLogURL, AllUnknownConditions, MarkResourcesUnavailable("Insufficient energy",
-					"Unschedulable"), withDefaultContainerStatuses(), withObservedGeneration(1)),
+			Object: Revision("foo", "pod-schedule-error",
+				WithLogURL, allUnknownConditions, MarkResourcesUnavailable("Insufficient energy",
+					"Unschedulable"), withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: pa("foo", "pod-schedule-error", WithReachabilityUnreachable),
@@ -510,16 +555,17 @@ func TestReconcile(t *testing.T) {
 		// Revision.  It then creates an Endpoints resource with active subsets.
 		// This signal should make our Reconcile mark the Revision as Ready.
 		Objects: []runtime.Object{
-			rev("foo", "steady-ready", withK8sServiceName("very-steady"), WithLogURL),
-			pa("foo", "steady-ready", WithTraffic, WithPAStatusService("steadier-even")),
+			Revision("foo", "steady-ready", WithK8sServiceName("very-steady"), WithLogURL),
+			pa("foo", "steady-ready", WithPASKSReady, WithTraffic,
+				WithScaleTargetInitialized, WithPAStatusService("steadier-even")),
 			deploy(t, "foo", "steady-ready"),
 			image("foo", "steady-ready"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "steady-ready", withK8sServiceName("steadier-even"), WithLogURL,
+			Object: Revision("foo", "steady-ready", WithK8sServiceName("steadier-even"), WithLogURL,
 				// All resources are ready to go, we should see the revision being
 				// marked ready
-				MarkRevisionReady, withDefaultContainerStatuses(), withObservedGeneration(1)),
+				MarkRevisionReady, withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "RevisionReady", "Revision becomes ready upon all resources being ready"),
@@ -529,17 +575,17 @@ func TestReconcile(t *testing.T) {
 		Name:    "lost pa owner ref",
 		WantErr: true,
 		Objects: []runtime.Object{
-			rev("foo", "missing-owners", withK8sServiceName("lesser-revision"), WithLogURL,
+			Revision("foo", "missing-owners", WithK8sServiceName("lesser-revision"), WithLogURL,
 				MarkRevisionReady),
 			pa("foo", "missing-owners", WithTraffic, WithPodAutoscalerOwnersRemoved),
 			deploy(t, "foo", "missing-owners"),
 			image("foo", "missing-owners"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "missing-owners", withK8sServiceName("lesser-revision"), WithLogURL,
+			Object: Revision("foo", "missing-owners", WithK8sServiceName("lesser-revision"), WithLogURL,
 				MarkRevisionReady,
 				// When we're missing the OwnerRef for PodAutoscaler we see this update.
-				MarkResourceNotOwned("PodAutoscaler", "missing-owners"), withDefaultContainerStatuses(), withObservedGeneration(1)),
+				MarkResourceNotOwned("PodAutoscaler", "missing-owners"), withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "InternalError", `revision: "missing-owners" does not own PodAutoscaler: "missing-owners"`),
@@ -549,17 +595,17 @@ func TestReconcile(t *testing.T) {
 		Name:    "lost deployment owner ref",
 		WantErr: true,
 		Objects: []runtime.Object{
-			rev("foo", "missing-owners", withK8sServiceName("youre-gonna-lose"), WithLogURL,
+			Revision("foo", "missing-owners", WithK8sServiceName("youre-gonna-lose"), WithLogURL,
 				MarkRevisionReady),
 			pa("foo", "missing-owners", WithTraffic),
 			noOwner(deploy(t, "foo", "missing-owners")),
 			image("foo", "missing-owners"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "missing-owners", withK8sServiceName("youre-gonna-lose"), WithLogURL,
+			Object: Revision("foo", "missing-owners", WithK8sServiceName("youre-gonna-lose"), WithLogURL,
 				MarkRevisionReady,
 				// When we're missing the OwnerRef for Deployment we see this update.
-				MarkResourceNotOwned("Deployment", "missing-owners-deployment"), withDefaultContainerStatuses(), withObservedGeneration(1)),
+				MarkResourceNotOwned("Deployment", "missing-owners-deployment"), withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "InternalError", `revision: "missing-owners" does not own Deployment: "missing-owners-deployment"`),
@@ -569,7 +615,7 @@ func TestReconcile(t *testing.T) {
 		Name: "image pull secrets",
 		// This test case tests that the image pull secrets from revision propagate to deployment and image
 		Objects: []runtime.Object{
-			rev("foo", "image-pull-secrets", WithImagePullSecrets("foo-secret")),
+			Revision("foo", "image-pull-secrets", WithImagePullSecrets("foo-secret")),
 		},
 		WantCreates: []runtime.Object{
 			pa("foo", "image-pull-secrets"),
@@ -577,9 +623,9 @@ func TestReconcile(t *testing.T) {
 			imagePullSecrets(image("foo", "image-pull-secrets"), "foo-secret"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: rev("foo", "image-pull-secrets",
+			Object: Revision("foo", "image-pull-secrets",
 				WithImagePullSecrets("foo-secret"),
-				WithLogURL, AllUnknownConditions, MarkDeploying("Deploying"), withDefaultContainerStatuses(), withObservedGeneration(1)),
+				WithLogURL, allUnknownConditions, MarkDeploying("Deploying"), withDefaultContainerStatuses(), WithRevisionObservedGeneration(1)),
 		}},
 		Key: "foo/image-pull-secrets",
 	}}
@@ -593,13 +639,16 @@ func TestReconcile(t *testing.T) {
 			podAutoscalerLister: listers.GetPodAutoscalerLister(),
 			imageLister:         listers.GetImageLister(),
 			deploymentLister:    listers.GetDeploymentLister(),
-			serviceLister:       listers.GetK8sServiceLister(),
 			resolver:            &nopResolver{},
 		}
 
 		return revisionreconciler.NewReconciler(ctx, logging.FromContext(ctx), servingclient.Get(ctx),
 			listers.GetRevisionLister(), controller.GetEventRecorder(ctx), r,
-			controller.Options{ConfigStore: &testConfigStore{config: ReconcilerTestConfig()}})
+			controller.Options{
+				ConfigStore: &testConfigStore{
+					config: ReconcilerTestConfig(),
+				},
+			})
 	}))
 }
 
@@ -615,7 +664,7 @@ func timeoutDeploy(deploy *appsv1.Deployment, message string) *appsv1.Deployment
 	deploy.Status.Conditions = []appsv1.DeploymentCondition{{
 		Type:    appsv1.DeploymentProgressing,
 		Status:  corev1.ConditionFalse,
-		Reason:  "ProgressDeadlineExceeded",
+		Reason:  v1.ReasonProgressDeadlineExceeded,
 		Message: message,
 	}}
 	return deploy
@@ -643,11 +692,11 @@ func deployImagePullSecrets(deploy *appsv1.Deployment, secretName string) *appsv
 	return deploy
 }
 
-func imagePullSecrets(image *caching.Image, secretName string) *caching.Image {
-	image.Spec.ImagePullSecrets = []corev1.LocalObjectReference{{
+func imagePullSecrets(Revision *caching.Image, secretName string) *caching.Image {
+	Revision.Spec.ImagePullSecrets = []corev1.LocalObjectReference{{
 		Name: secretName,
 	}}
-	return image
+	return Revision
 }
 
 func changeContainers(deploy *appsv1.Deployment) *appsv1.Deployment {
@@ -658,46 +707,9 @@ func changeContainers(deploy *appsv1.Deployment) *appsv1.Deployment {
 	return deploy
 }
 
-func rev(namespace, name string, ro ...RevisionOption) *v1.Revision {
-	r := &v1.Revision{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:       name,
-			Namespace:  namespace,
-			UID:        "test-uid",
-			Generation: 1,
-		},
-		Spec: v1.RevisionSpec{
-			PodSpec: corev1.PodSpec{
-				Containers: []corev1.Container{{
-					Name:  name,
-					Image: "busybox",
-				}},
-			},
-		},
-	}
-	r.SetDefaults(context.Background())
-
-	for _, opt := range ro {
-		opt(r)
-	}
-	return r
-}
-
-func withK8sServiceName(sn string) RevisionOption {
-	return func(r *v1.Revision) {
-		r.Status.ServiceName = sn
-	}
-}
-
-func withObservedGeneration(gen int64) RevisionOption {
-	return func(r *v1.Revision) {
-		r.Status.ObservedGeneration = gen
-	}
-}
-
 func withDefaultContainerStatuses() RevisionOption {
 	return func(r *v1.Revision) {
-		r.Status.ContainerStatuses = []v1.ContainerStatuses{{
+		r.Status.ContainerStatuses = []v1.ContainerStatus{{
 			Name:        r.Name,
 			ImageDigest: "",
 		}}
@@ -705,7 +717,7 @@ func withDefaultContainerStatuses() RevisionOption {
 }
 
 // TODO(mattmoor): Come up with a better name for this.
-func AllUnknownConditions(r *v1.Revision) {
+func allUnknownConditions(r *v1.Revision) {
 	WithInitRevConditions(r)
 	MarkDeploying("")(r)
 	MarkActivating("Deploying", "")(r)
@@ -723,7 +735,7 @@ func deploy(t *testing.T, namespace, name string, opts ...interface{}) *appsv1.D
 		}
 	}
 
-	rev := rev(namespace, name)
+	rev := Revision(namespace, name)
 
 	for _, opt := range opts {
 		if revOpt, ok := opt.(RevisionOption); ok {
@@ -748,11 +760,11 @@ func image(namespace, name string, co ...configOption) *caching.Image {
 		opt(config)
 	}
 
-	return resources.MakeImageCache(rev(namespace, name), name, "")
+	return resources.MakeImageCache(Revision(namespace, name), name, "")
 }
 
 func pa(namespace, name string, ko ...PodAutoscalerOption) *asv1a1.PodAutoscaler {
-	rev := rev(namespace, name)
+	rev := Revision(namespace, name)
 	k := resources.MakePA(rev)
 
 	for _, opt := range ko {
@@ -791,14 +803,14 @@ var _ pkgreconciler.ConfigStore = (*testConfigStore)(nil)
 
 func ReconcilerTestConfig() *config.Config {
 	return &config.Config{
-		Deployment: getTestDeploymentConfig(),
+		Deployment: testDeploymentConfig(),
 		Observability: &metrics.ObservabilityConfig{
 			LoggingURLTemplate: "http://logger.io/${REVISION_UID}",
 		},
 		Logging:  &logging.Config{},
 		Tracing:  &tracingconfig.Config{},
 		Defaults: &defaultconfig.Defaults{},
-		Autoscaler: &asconfig.Config{
+		Autoscaler: &autoscalerconfig.Config{
 			InitialScale:          1,
 			AllowZeroInitialScale: false,
 		},
